/*	xdmp.c  v2.4  11/14/91	*/
/*	(c) lane watts 1991		*/

#include <stdio.h>
#include <fcntl.h>
#include <io.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <time.h>

#if !defined(fileno)
#define fileno(f)	    ((f)->fd)
#endif

#if	!defined(TRUE) || ((TRUE) != 1)
#define	TRUE	(1)
#endif
#if	!defined(FALSE) || ((FALSE) != 0)
#define	FALSE	(0)
#endif

#define REV "2.4"

#define BFSIZE 512

#define LPP 60

extern int errno;

int	ercd;
int	pgnate;
int	fl1;
int	lns,	pg,	pg1;

char	pgmbsnm[81];
char	timestr[32];
char	hexstr[50];
char	charstr[18];

time_t	tm;

struct	dumpspec {
	char flnm[81];
	long st;
	long ln;
	long nd;
};

void prtusg(void);						/* print usage message				*/
struct dumpspec *getspecs(char *spex);	/* get specs for file to dump		*/
void dmpfl(struct dumpspec *d);			/* produce dump of one file			*/
void bldstr(int pos, char chr);			/* build hex and ascii strings		*/
void prthdr(struct dumpspec *d);		/* print header						*/
char *basename(char *str);				/* strip dir & ext from filename	*/

void main(int argc,char *argv[])
{
	int i;
	int ef;
	char c;

	ercd = 0; pg1 = TRUE; fl1 = 0; ef = FALSE;
	strncpy(pgmbsnm,basename(argv[0]),sizeof(pgmbsnm)-1);
	pgmbsnm[sizeof(pgmbsnm)-1] = '\0';

	pgnate = FALSE;
	for (i=1; i<argc; i++) {
		if (argv[i][0] == '-') {
			switch (argv[i][1]) {
			case 'p':	c = argv[i][2];
						pgnate = ((c == '+') || (c == '\0')) ? TRUE : FALSE;
						break;
			case 'h':
			case '?':
			default:	prtusg();
						ef = TRUE;
						break;
			}
		} else {
			dmpfl(getspecs(argv[i]));
			ef = TRUE;
			fl1 = 1;
		}
	}
	if ((!fl1) && (ef == FALSE))
		dmpfl(getspecs(""));

    exit(ercd);
}

void prtusg()							/* print usage message				*/
{
	printf("\n");
	printf("%s (rev %s) - produce a listing of a file in hexadecimal and ascii\n",pgmbsnm,REV);
	printf("usage:\n");
	printf("    %s [options] filename,s,c,e [filename,s,c,e ...]\n",pgmbsnm);
	printf("where options are:\n");
	printf(" -h        = display this list of parameters\n");
	printf(" -?        = display this list of parameters\n");
	printf(" -p        = paginate output\n");
	printf("and:\n");
	printf("  filename = name of the file to be listed\n");
	printf("  s        = byte position within the file to start the listing\n");
	printf("  c        = number of bytes to list\n");
	printf("             if e is specified, c is ignored.\n");
	printf("  e        = byte position within the file to end the listing\n");
	printf("the listing will always go to the standard output file (stdout).\n");
	printf("stdin may be used by leaving the file name blank.\n");
	printf("s, c, and e may be omitted and will default as follows:\n");
	printf("if s is omitted, it will default to 0\n");
	printf("if c is omitted, it will default to end-start\n");
	printf("if e is omitted, it will default to start+count\n");
	printf("if c and e are omitted, it will default to the end of the file\n");
	printf("\n");
	return;
}

struct dumpspec *getspecs(char spex[])	/* get specs for file to dump		*/
{
	int 	i;
	char	spstr[81];
	char	*ps1,	*ps2;
	static struct dumpspec ds;

	memset(&ds,'\0',sizeof(ds));
	strncpy(spstr,spex,sizeof(spstr)-1);
	ps1 = spstr;
	for (i=0; ((i<4) && (ps1 != (char *)0)); i++) {
		ps2 = strchr(ps1,',');
		if (ps2 != (char *)0) {
			*ps2 = '\0';
			ps2++;
		}
		switch (i) {
		case 0:	strcpy(ds.flnm,ps1);
			break;
		case 1:	ds.st = atol(ps1);
			break;
		case 2:	ds.ln = atol(ps1);
			break;
		case 3:	ds.nd = atol(ps1);
			break;
		}
		ps1 = ps2;
	}
	if ((ds.ln == 0) && (ds.nd != 0))
		ds.ln = (ds.nd+1) - ds.st;
	if ((ds.nd == 0) && (ds.ln != 0))
		ds.nd = (ds.st + ds.ln) - 1;

	return(&ds);
}

void dmpfl(struct dumpspec *d)			/* produce dump of one file			*/
{
	int	j,	k;
	int	bp;
	int	cnt,	tcnt;

	char	inbuf[BFSIZE];
	char	*pc;
	FILE	*fp;

	lns = LPP+1; pg = 0;
	tm = time(0);
	strncpy(timestr,ctime(&tm),sizeof(timestr));
	pc = strchr(timestr,'\n');			/* replace \n w/ null				*/
	if (pc != (char *)0)
		*pc = '\0';

	if ((int)strlen(d->flnm) <= 0) {
		fp = stdin;
		if (setmode(fileno(stdin),O_BINARY) == -1) {
			sprintf((char *)inbuf,"error setting file mode on stdin");
			perror((char *)inbuf);
			printf("\n\n");
			ercd = errno;
			return;
		}
		strcpy(d->flnm,"(stdin)");
	} else {
		if ((fp = fopen(d->flnm,"rb")) == (FILE *)0) {
			sprintf((char *)inbuf,"error opening file %s",d->flnm);
			perror((char *)inbuf);
			printf("\n\n");
			ercd = errno;
			return;
		}
	}

	if (fseek(fp,d->st,0) != 0) {
		sprintf((char *)inbuf,"error reading byte %d of %s\007",d->st,d->flnm);
		perror((char *)inbuf);
		printf("\n\n");
		ercd = errno;
		return;
	}

	k = d->st % 16;
	tcnt = d->st-k;
	cnt = -1; bp = 0;
	do {
		if ((d->ln > 0) && (tcnt > d->nd))
			break;
		memset(hexstr,' ',49); hexstr[49] = '\0';
		memset(charstr,' ',16); charstr[16] = '\0';
		for (j=k; j<16; j++) {
			if (bp >= cnt) {
				cnt = fread((char *)inbuf,sizeof(*inbuf),BFSIZE,fp);
				bp = 0;
				if (cnt <= 0)
					break;
			}
			if ((d->ln > 0) && ((tcnt+j) > d->nd))
				break;
			bldstr(j,inbuf[bp++]);
		}
		lns++;
		if (lns > LPP) {
			if (pgnate || (pg == 0))
				prthdr(d);
		}
		printf("%6.6d |%s |%s|\n",tcnt,hexstr,charstr);
		tcnt += 16;
		k = 0;
		if ((d->ln > 0) && (tcnt > d->nd))
			break;
	} while (cnt > 0);

	if (!pgnate)
		printf("\n\n\n");
	fclose(fp);
	return;
}

void bldstr(int pos, char chr)			/* build hex and ascii strings		*/
{
	int	offset;
	unsigned	uns;
	char	tmp[20];
	char	nohi;

	offset = (pos<8) ? (pos*3)+1 : (pos*3)+2;
	uns = (unsigned)(chr & 255);
	sprintf((char *)tmp,"%2.2x",uns);
	memcpy(&hexstr[offset],(char *)tmp,2);
	nohi = (unsigned)(uns & (unsigned)127);
	charstr[pos] = (nohi>=32 && nohi<127) ? nohi : '.';
}

void prthdr(struct dumpspec *d)			/* print header						*/
{
	if ((!pg1) && pgnate)
		printf("\f");
	else
		(pg1 = FALSE);
	pg++; lns = 0;

	printf("%s         xdmp r%s - hex/ascii dump          page: %d\n",timestr,REV,pg);
	printf("file: %s  start: %ld  count: %ld  end: %ld\n\n",d->flnm,d->st,d->ln,d->nd);
	return;
}

char *basename(char *str)				/* strip dir & ext from filename	*/
{
	char *ps1, *ps2;
	static char tstr[81];

	strncpy(tstr,str,sizeof(tstr)-1);
	tstr[sizeof(tstr)-1] = '\0';
	ps1 = strrchr(tstr,'\\');
	if (ps1 == (char *)0)
		ps1 = tstr;
	else
		ps1++;
	ps2 = strchr(ps1,'.');
	if (ps2 != (char *)0)
		*ps2 = '\0';
	return(ps1);
}

